@@ -10,8 +10,8 @@ class TextTagsEvalItem(EvalItem):
     SHAPE_KEY = 'labels'
 
     def spans_iou(self, x, y):
-        s1, e1 = x['start'], x['end']
-        s2, e2 = y['start'], y['end']
+        s1, e1 = int(x['start']), int(x['end'])
+        s2, e2 = int(y['start']), int(y['end'])
         if s2 > e1 or s1 > e2:
             return 0
         intersection = min(e1, e2) - max(s1, s2)
@@ -30,10 +30,10 @@ class TextTagsEvalItem(EvalItem):
         comparator = get_text_comparator(algorithm, qval)
         label_weights = label_weights or {}
         someone_is_empty = self.empty ^ item.empty
-        if someone_is_empty:
+        if someone_is_empty and not per_label:
             return 0
         if self.empty and item.empty:
-            return 1
+            return {} if per_label else 1
 
         gt_values = self.get_values()
         if per_label:
@@ -41,11 +41,15 @@ class TextTagsEvalItem(EvalItem):
         else:
             total_score, total_weight = 0, 0
         for pred_value in item.get_values_iter():
-            # find the best matching span inside gt_values
-            best_matching_score = max(map(partial(self._match, y=pred_value, f=comparator), gt_values))
-            if iou_threshold is not None:
-                # make hard decision w.r.t. threshold whether current spans are matched
-                best_matching_score = float(best_matching_score > iou_threshold)
+            if len(gt_values) == 0:
+                # for empty gt values, matching score for current prediction is the lowest
+                best_matching_score = 0
+            else:
+                # find the best matching span inside gt_values
+                best_matching_score = max(map(partial(self._match, y=pred_value, f=comparator), gt_values))
+                if iou_threshold is not None:
+                    # make hard decision w.r.t. threshold whether current spans are matched
+                    best_matching_score = float(best_matching_score > iou_threshold)
             if per_label:
                 # for per-label mode, label weights are unimportant - only scores are averaged
                 for l in pred_value[self._shape_key]:
@@ -89,6 +93,12 @@ class HTMLTagsEvalItem(TextTagsEvalItem):
         if s2 > e1 or s1 > e2:
             return 0
 
+        # correct end offset if they lie in different block
+        # if e1 <= s1:
+        #     e1 = s1 + len(x.get('text', ''))
+        # if e2 <= s2:
+        #     e2 = s2 + len(y.get('text', ''))
+
         intersection = min(e1, e2) - max(s1, s2)
         union = max(e1, e2) - min(s1, s2)
         if union == 0:
